namespace Tiaoxinv1_raw {
int numRounds = -1;

/*
 * AES round Reference C Implementation
 * 
 * Copyright 2014: 
 *     Ivica Nikolic <cube444@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */

#include <string.h>
#include "tiaoxin-reference.h"

/*
 * The S-boxes of AES
 */
unsigned char sbox[256] = 
{
   0x63, 0x7C, 0x77, 0x7B, 0xF2, 0x6B, 0x6F, 0xC5, 0x30, 0x01, 0x67, 0x2B, 0xFE, 0xD7, 0xAB, 0x76,
   0xCA, 0x82, 0xC9, 0x7D, 0xFA, 0x59, 0x47, 0xF0, 0xAD, 0xD4, 0xA2, 0xAF, 0x9C, 0xA4, 0x72, 0xC0,
   0xB7, 0xFD, 0x93, 0x26, 0x36, 0x3F, 0xF7, 0xCC, 0x34, 0xA5, 0xE5, 0xF1, 0x71, 0xD8, 0x31, 0x15,
   0x04, 0xC7, 0x23, 0xC3, 0x18, 0x96, 0x05, 0x9A, 0x07, 0x12, 0x80, 0xE2, 0xEB, 0x27, 0xB2, 0x75,
   0x09, 0x83, 0x2C, 0x1A, 0x1B, 0x6E, 0x5A, 0xA0, 0x52, 0x3B, 0xD6, 0xB3, 0x29, 0xE3, 0x2F, 0x84,
   0x53, 0xD1, 0x00, 0xED, 0x20, 0xFC, 0xB1, 0x5B, 0x6A, 0xCB, 0xBE, 0x39, 0x4A, 0x4C, 0x58, 0xCF,
   0xD0, 0xEF, 0xAA, 0xFB, 0x43, 0x4D, 0x33, 0x85, 0x45, 0xF9, 0x02, 0x7F, 0x50, 0x3C, 0x9F, 0xA8,
   0x51, 0xA3, 0x40, 0x8F, 0x92, 0x9D, 0x38, 0xF5, 0xBC, 0xB6, 0xDA, 0x21, 0x10, 0xFF, 0xF3, 0xD2,
   0xCD, 0x0C, 0x13, 0xEC, 0x5F, 0x97, 0x44, 0x17, 0xC4, 0xA7, 0x7E, 0x3D, 0x64, 0x5D, 0x19, 0x73,
   0x60, 0x81, 0x4F, 0xDC, 0x22, 0x2A, 0x90, 0x88, 0x46, 0xEE, 0xB8, 0x14, 0xDE, 0x5E, 0x0B, 0xDB,
   0xE0, 0x32, 0x3A, 0x0A, 0x49, 0x06, 0x24, 0x5C, 0xC2, 0xD3, 0xAC, 0x62, 0x91, 0x95, 0xE4, 0x79,
   0xE7, 0xC8, 0x37, 0x6D, 0x8D, 0xD5, 0x4E, 0xA9, 0x6C, 0x56, 0xF4, 0xEA, 0x65, 0x7A, 0xAE, 0x08,
   0xBA, 0x78, 0x25, 0x2E, 0x1C, 0xA6, 0xB4, 0xC6, 0xE8, 0xDD, 0x74, 0x1F, 0x4B, 0xBD, 0x8B, 0x8A,
   0x70, 0x3E, 0xB5, 0x66, 0x48, 0x03, 0xF6, 0x0E, 0x61, 0x35, 0x57, 0xB9, 0x86, 0xC1, 0x1D, 0x9E,
   0xE1, 0xF8, 0x98, 0x11, 0x69, 0xD9, 0x8E, 0x94, 0x9B, 0x1E, 0x87, 0xE9, 0xCE, 0x55, 0x28, 0xDF,
   0x8C, 0xA1, 0x89, 0x0D, 0xBF, 0xE6, 0x42, 0x68, 0x41, 0x99, 0x2D, 0x0F, 0xB0, 0x54, 0xBB, 0x16
};

/*
 * The MixColumns matrix of AES
 */
unsigned char MC[4][4] = { {2,3,1,1}, {1,2,3,1}, {1,1,2,3}, {3,1,1,2} };

/*
 * The SubBytes transformation of AES
 */
void SubBytes( unsigned char state[4][4] )
{
  unsigned int i,j;
  
  for(i=0; i<4; i++)
    for(j=0; j<4; j++)
       state[i][j] = sbox[ (unsigned int)state[i][j] ];   
    
}

/*
 * The ShiftRows transformation of AES 
 */
void ShiftRows( unsigned  char state[4][4] )
{
  int i,j;
  unsigned char tmp[4][4];
  
  for(i=0; i<4; i++){
    for(j=0; j<4; j++)
      tmp[i][j] = state[i][(j+i)%4];
  }
  memcpy( state, tmp, 4*4 );  
}

/* 
 * The field multiplication used in the MixColumns
 */
unsigned char FieldMult(unsigned char a, unsigned char b)
{  
  const unsigned char ReductionPoly = 0x1b;
  unsigned char x = a, ret = 0;
  int i;
  for(i = 0; i < 8; i++) {
	  if((b>>i)&1) ret ^= x;
	  if(x&0x80) { x <<= 1; x ^= ReductionPoly; }
	  else x <<= 1;
  }
  return ret;  
}

/*
 * The MixColumns transformation of AES 
 */
void MixColumns( unsigned char state[4][4] )
{
  int col,row,k;
  unsigned char tmp[4][4];
  
  memset ( tmp, 0, 4*4 );
  
  for(col=0; col<4; col++){    
    for(row=0; row<4; row++){
      for(k=0; k<4; k++)
	tmp[row][col] ^= FieldMult( MC[row][k], state[k][col] );
    }    
  }
  memcpy( state, tmp, 4*4 );
}

/* 
 * One AES round
 */
void AES_round( word P, word K, word C)
{
  int i,j;
  unsigned char state[4][4];
  
  // Convert the 16 byte sequence to a 4x4 matrix
  for(i=0; i<4; i++)
    for(j=0; j<4; j++)
      state[i][j] = P[ 4 * j + i ];
  
  // Apply keyless AES round
  SubBytes  ( state );
  ShiftRows ( state );
  MixColumns( state );
  
  // Convert the 4x4 matrix to 16-byte sequence 
  for(i=0; i<4; i++)
    for(j=0; j<4; j++)
      C[ 4 * j + i ] = state[i][j];
    
  // Xor the key
  for(i=0; i<16; i++)
    C[i] ^= K[i];
}
































} // namespace Tiaoxinv1_raw
